local addonName, a = ...

local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local GetTime = GetTime
local GetTotemInfo = GetTotemInfo
local IsSwimming = IsSwimming
local math = math

local function shouldFlashImbue(buffTooltipName, offhand)
	local min
	if s.InCombat() then
		min = 0
	elseif s.InRaidOrParty() then
		min = 5 * 60
	else
		min = 2 * 60
	end
	if offhand then
		return not s.OffHandItemBuff(L[buffTooltipName], min)
	else
		return not s.MainHandItemBuff(L[buffTooltipName], min)
	end
end

local function missingTotem(slot)
	local haveTotem, name, startTime, duration, icon = GetTotemInfo(slot)
	duration = startTime + duration - GetTime() - c.GetBusyTime()
	return duration <= 0
end

------------------------------------------------------------------------ Common
c.AddOptionalSpell("Lightning Shield", nil, {
	Override = function()
		return c.GetBuffStack("Lightning Shield") < 2
			and c.SelfBuffNeeded("Lightning Shield")
	end
})

c.AddOptionalSpell("Ascendance")

c.AddOptionalSpell("Ancestral Swiftness", nil, {
	CheckFirst = function()
		return not c.HasBuff("Ascendance")
	end
})

c.AddOptionalSpell("Earth Elemental Totem", nil, {
	CheckFirst = function()
		return c.GetCooldown("Fire Elemental Totem") > 50
	end
})

c.AddOptionalSpell("Fire Elemental Totem", nil, {
	CheckFirst = function()
		return missingTotem(1)
	end
})

c.AddOptionalSpell("Elemental Mastery")

c.AddSpell("Elemental Blast", nil, {
	NotIfActive = true,
})

c.AddOptionalSpell("Searing Totem", nil, {
	CheckFirst = function()
		return missingTotem(1)
	end
})

c.AddOptionalSpell("Healing Surge", "when Solo", {
	Override = function()
		return not s.InRaidOrParty() and s.HealthPercent("player") < 80
	end
})

c.AddOptionalSpell("Water Walking", nil, {
	Override = function()
		return IsSwimming() and c.SelfBuffNeeded("Water Walking")
	end
})

c.AddInterrupt("Wind Shear")

--------------------------------------------------------------------- Elemental
c.AddOptionalSpell("Elemental Mastery", "unless Bloodlust", {
	CheckFirst = function()
		return not c.HasBuff(c.BLOODLUST_BUFFS)
	end
})

c.AddOptionalSpell("Ascendance", "for Elemental", {
	NotWhileActive = true,
	CheckFirst = function()
		return c.GetMyDebuffDuration("Flame Shock") >= 15
			and c.GetCooldown("Lava Burst") > 0
	end
})

c.AddSpell("Flame Shock", "for Elemental", {
	EarlyRefresh = 0,
	NotIfActive = true,
	CheckFirst = function(z)
		if c.HasBuff("Ascendance") then
			return false
		end
		
		local duration = c.GetMyDebuffDuration("Flame Shock")
		if c.HasBuff("Elemental Mastery") or c.HasBuff(c.BLOODLUST_BUFFS) then
			return duration <= 2 * z.EarlyRefresh
		else
			return duration <= z.EarlyRefresh
		end
	end
})
c.ManageDotRefresh("Flame Shock for Elemental", 3, "Flame Shock")

c.AddSpell("Lava Burst", nil, {
	CheckFirst = function()
		return (c.GetMyDebuffDuration("Flame Shock")
					> c.GetCastTime("Lava Burst") + .1
				or c.IsAuraPendingFor("Flame Shock"))
			and (not c.IsCasting("Lava Burst")
				or (c.HasBuff("Lava Surge") and not c.IsQueued("Lava Burst")) 
				or c.HasBuff("Ascendance"))
	end
})

c.AddSpell("Earth Shock", "for Fulmination", {
	CheckFirst = function()
        local stacks = s.BuffStack(c.GetID("Lightning Shield"), "player")
        if stacks == 7 then
        	return true
        end
        
        local remaining = c.GetMyDebuffDuration("Flame Shock")
        return stacks > 3 and (remaining > 6 and remaining < 8)
	end
})

c.AddSpell("Unleash Elements", "for Elemental", {
	CheckFirst = function()
		return c.HasTalent("Unleashed Fury") and not c.HasBuff("Ascendance")
	end
})

c.AddOptionalSpell("Searing Totem", "for Elemental", {
	CheckFirst = function()
		return missingTotem(1) and c.GetCooldown("Fire Elemental Totem") > 15
	end
})

c.AddOptionalSpell("Thunderstorm", nil, {
	CheckFirst = function()
		return s.PowerPercent("player") < 85
	end
})

c.AddOptionalSpell("Flametongue Weapon", nil, {
	FlashID = { "Flametongue Weapon", "Weapon Imbues" },
	Override = function()
		return shouldFlashImbue("Flametongue")
	end
})

----------------------------------------------------------------------- Enhance
c.AddOptionalSpell("Windfury Weapon", nil, {
	FlashID = { "Windfury Weapon", "Weapon Imbues" },
	Override = function()
		return shouldFlashImbue("Windfury")
	end
})

c.AddOptionalSpell("Flametongue Weapon", "Offhand", {
	FlashID = { "Flametongue Weapon", "Weapon Imbues" },
	Override = function()
		return shouldFlashImbue("Flametongue", true)
	end
})

c.AddSpell("Unleash Elements", "with Unleashed Fury", {
	CheckFirst = function()
		return c.HasTalent("Unleashed Fury")
	end
})

c.AddSpell("Stormstrike", nil, {
	FlashID = { "Stormblast", "Stormstrike" },
	CheckFirst = function(z)
		if c.HasBuff("Ascendance") then
			z.ID = c.GetID("Stormblast")
		else
			z.ID = c.GetID("Stormstrike")
		end
		return true
	end
})

c.AddSpell("Lightning Bolt", "at 5", {
	CheckFirst = function()
		return a.Maelstrom == 5
	end
})

c.AddSpell("Lightning Bolt", "at 4", {
	NotWhileMoving = true,
	CheckFirst = function()
		return a.Maelstrom > 3 and not c.HasBuff("Ascendance")
	end
})

c.AddSpell("Lightning Bolt", "at 2", {
	NotWhileMoving = true,
	CheckFirst = function()
		return a.Maelstrom > 1 and not c.HasBuff("Ascendance")
	end
})

c.AddSpell("Lightning Bolt", "under Ancestral Swiftness", {
	CheckFirst = function()
		return c.HasBuff("Ancestral Swiftness")
	end
})

c.AddOptionalSpell("Ancestral Swiftness", "under 2", {
	CheckFirst = function()
		return c.GetBuffStack("Maelstrom Weapon") < 2
	end
})

c.AddSpell("Flame Shock", "under Unleash Flame", {
	MyDebuff = "Flame Shock",
	CheckFirst = function()
		return c.HasBuff("Unleash Flame")
	end
})
c.ManageDotRefresh("Flame Shock under Unleash Flame", 3, "Flame Shock")

c.AddSpell("Flame Shock", "Overwrite under Unleash Flame", {
	CheckFirst = function()
		return c.HasBuff("Unleash Flame")
	end
})

c.AddOptionalSpell("Feral Spirit", nil, {
	NoRangeCheck = true,
})

------------------------------------------------------------------------- Resto
c.AddOptionalSpell("Earthliving Weapon", nil, {
	FlashID = { "Earthliving Weapon", "Weapon Imbues" },
	Override = function()
		return shouldFlashImbue("Earthliving")
	end
})

c.AddOptionalSpell("Water Shield", nil, {
	Override = function()
		return c.SelfBuffNeeded("Water Shield")
	end
})

c.AddOptionalSpell("Earth Shield", nil, {
	Override = function(z)
		if not s.InRaidOrParty() then
			return false
		end
		
		if not a.EarthShieldTarget then
			return true
		end
		
		if s.InCombat() then
			return not s.MyBuff(z.ID, a.EarthShieldTarget)
		else
			return s.MyBuffDuration(z.ID, a.EarthShieldTarget) < 2 * 60
				or s.MyBuffStack(z.ID, a.EarthShieldTarget) < 9
		end
	end
})

c.AddOptionalSpell("Healing Stream Totem", nil, {
	CheckFirst = function()
		return missingTotem(3)
	end
})

c.AddOptionalSpell("Mana Tide Totem", nil, {
	CheckFirst = function()
		return missingTotem(3) and s.PowerPercent("player") < 75
	end
})

c.AddOptionalSpell("Lightning Bolt", "for Mana", {
	Override = function()
		return c.HasGlyph("Telluric Currents")
			and s.PowerPercent("player") < 95
			and s.HealthPercent("raid|range") > 90
	end
})