local addon_name, addon = ...

local RedirectOptions = {
	["Default"] = "Default",
	["Chatframe"] = "Chatframe",
	["Raidwarning"] = "Raidwarning",
	["Bossemote"] = "Bossemote",
	["MSBT"] = "MSBT",
	["SCT"] = "SCT",
	["Parrot"] = "Parrot",
	["BlizzardCT"] = "BlizzardCT"
}

local function RGBPercToHex(r, g, b, a)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function HexToRGBPerc(hex)
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255, 1
end

local options
function addon:GetOptions()
	default = addon:GetDefault()
	options = {
		name = "ErrorHandler",
		handler = addon,
		type = "group",
		childGroups = "tab",
		order = 1,
		args = {
			reset = {
				name = "Reset",
				desc = "Reset the options to default",
				type = "execute",
				func = function()
					self:ResetOptions()
				end,
			},
		},
	}
	for k,v in ipairs(default.ErrCat) do
		options.args[tostring(k)] = {
			name = v.ErrCatName,
			order = k,
			type = "group",
			args = {}
			}
	end
	for k,v in pairs(default.ErrName) do
		options.args[tostring(v.ErrCat)].args[k] = {
							name = k,
							desc = k,
							type = "group",
							inline = true,
							args = {
							[k.."Hide"] = {
								name = "Hide",
								desc = "Toggle the visibility of the error",
								type = "toggle",
								order = 0,
								get = function() return ErrDB.ErrName[k].ErrHide end,
								set = function(_, value)
									ErrDB.ErrName[k].ErrHide = value
								end,
								},
							[k.."Redirect"] = {
								name = "Redirection",
								desc = "If shown, redirection of the error",
								type = "select",
								style = "dropdown",
								values = RedirectOptions,
								get = function() return ErrDB.ErrName[k].ErrRdr end,
								set = function(_, value)
									ErrDB.ErrName[k].ErrRdr = value
								end,
								},							
							[k.."Color"] = {
								name = "Color",
								desc = "If shown, color of the error",
								type = "color",
								get = function() return HexToRGBPerc(ErrDB.ErrName[k].ErrColor) end,
								set = function(_, r, g, b, a)
									ErrDB.ErrName[k].ErrColor = RGBPercToHex(r, g, b, a)
								end,
								},
							},
		}
	end
	return options
end
